#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>
#include <sys/fcntl.h>
#include <mailer.h>

#define NR_ARGS  0x10            /* Number of args for 'system()' */
#define FAIL -1

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/*
 *   This gets the return status of any child processes and throws it
 *   away. In this manner, we don't retain any zombies.
 */

void reaper(int unused)
{
    while(wait3(&unused, WNOHANG, NULL) > 0)
          ;
}
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/*
 *  Common routine to set up the signal handlers.
 */
void set_sig(int sig, sig_t func, int flags)
{
    struct sigaction sa;
    if(sigaction(sig, NULL, &sa) == FAIL)
        ERRORS(errno);
    sa.sa_flags   = flags;
    sa.sa_handler = func;
    if(sigaction(sig, &sa, NULL) == FAIL)
        ERRORS(errno);
    return;
}

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
 */
void execute_command(const char *path)
{
    char cmd[0x100];
    size_t i;
    char *args[NR_ARGS];
    char *p;

    bzero(cmd, sizeof(cmd));
    strncpy(cmd, path, sizeof(cmd));
    i = 0;
    if((p = strtok(cmd, " ")) == NULL)
        p = cmd;
    args[i++] = p;
    for(; i< NR_ARGS; i++)
        if((args[i] = strtok(NULL, " ")) == NULL)
            break;
    set_sig(SIGCHLD, reaper, SA_INTERRUPT|SA_RESTART);
    switch(fork())
    {
    case 0:
        set_sig(SIGINT,  SIG_IGN, SA_INTERRUPT|SA_RESTART);
        set_sig(SIGQUIT, SIG_IGN, SA_INTERRUPT|SA_RESTART);
        setsid();
        set_sig(SIGCHLD, reaper, SA_INTERRUPT|SA_RESTART);
        execve(args[0], args, __environ);
        exit(EXIT_FAILURE);
        break;
    case -1:
        return;
    default:
        break;
    }
}
